#include <Arduino.h>
#include <SPI.h>
#include <VarioSettings.h>

#include <I2Cdev.h>
#include <ms5611.h>
#include <vertaccel.h>
#include <EEPROM.h>
#include <LightInvensense.h>

#include <beeper.h>
#include <toneAC.h>
#include <avr/pgmspace.h>
#include <varioscreen.h>
#include <digit.h>

#include <SerialNmea.h>
#include <NmeaParser.h>

#include <FirmwareUpdater.h>

/***************/
/* IMU objects */
/***************/
#ifdef HAVE_ACCELEROMETER
#ifdef IMU_CALIBRATION_IN_EEPROM
VertaccelSettings vertaccelSettings = Vertaccel::readEEPROMSettings();
#else //!IMU_CALIBRATION_IN_EEPROM
const VertaccelSettings vertaccelSettings = {
  IMU_GYRO_CAL_BIAS
  ,{ IMU_ACCEL_CAL_BIAS, IMU_ACCEL_CAL_SCALE }
#ifdef AK89xx_SECONDARY
  ,{ IMU_MAG_CAL_BIAS, IMU_MAG_CAL_PROJ_SCALE }
#endif //AK89xx_SECONDARY
};
#endif //!IMU_CALIBRATION_IN_EEPROM

Vertaccel vertaccel(vertaccelSettings);
#endif //HAVE_ACCELEROMETER




#define VARIOSCREEN_ALTI_ANCHOR_X 52
#define VARIOSCREEN_ALTI_ANCHOR_Y 0
#define VARIOSCREEN_VARIO_ANCHOR_X 52
#define VARIOSCREEN_VARIO_ANCHOR_Y 2

VarioScreen screen(VARIOSCREEN_DC_PIN,VARIOSCREEN_CS_PIN,VARIOSCREEN_RST_PIN);
ScreenDigit altiDigit(screen, VARIOSCREEN_ALTI_ANCHOR_X, VARIOSCREEN_ALTI_ANCHOR_Y, 0);
ScreenDigit varioDigit(screen, VARIOSCREEN_VARIO_ANCHOR_X, VARIOSCREEN_VARIO_ANCHOR_Y, 0);
ScreenDigit countDigit(screen, VARIOSCREEN_VARIO_ANCHOR_X, VARIOSCREEN_VARIO_ANCHOR_Y + 2, 0);

ScreenSchedulerObject displayList[] = { {&altiDigit, 0}, {&varioDigit, 0}, {&countDigit, 0} };
ScreenScheduler varioScreen(screen, displayList, sizeof(displayList)/sizeof(ScreenSchedulerObject), 0, 0);

NmeaParser nmeaParser;

unsigned gpsCount = 0;

void setup() {

  /*****************************/
  /* wait for devices power on */
  /*****************************/
  delay(VARIOMETER_POWER_ON_DELAY);

  /**********************/
  /* init accelerometer */
  /**********************/
  Fastwire::setup(FASTWIRE_SPEED, 0);
#ifdef HAVE_ACCELEROMETER
  vertaccel.init();
  if( firmwareUpdateCond() ) {
   firmwareUpdate();
  }
#endif //HAVE_ACCELEROMETER

  /************/
  /* init SPI */
  /************/ 

  /* set all SPI CS lines before talking to devices */
  screen.enableSPI();

  /***************/
  /* init screen */
  /***************/
  screen.begin(VARIOSCREEN_CONTRAST);

   /**************************/
  /* init gps and bluetooth */
  /**************************/
  serialNmea.begin(GPS_BLUETOOTH_BAUDS, true);

}

void loop() {

  /**************/
  /* update GPS */
  /**************/

  /* try to lock sentences */
  if( serialNmea.lockRMC() ) {
    nmeaParser.beginRMC();
    gpsCount++;
    countDigit.setValue( gpsCount );
  } else if( serialNmea.lockGGA() ) {
    nmeaParser.beginGGA();
  }

  /* parse if needed */
  if( nmeaParser.isParsing() ) {
    while( nmeaParser.isParsing() ) {
      uint8_t c = serialNmea.read();
        
      /* parse sentence */        
      nmeaParser.feed( c );
    }

    serialNmea.release();
  }

  /*****************/
  /* update screen */
  /*****************/

  altiDigit.setValue( nmeaParser.satelliteCount );
  varioDigit.setValue( nmeaParser.precision );

  /* screen update */
  varioScreen.displayStep();
  

}
